/*------------------------------------------------------------------------------
* rtkcmn.h : rtk common types and functions
*
*          Copyright (C) 2006 by T.TAKASU, All rights reserved.
*
* version : $Revision: 1.2 $ $Date: 2006/12/12 23:27:39 $
* history : 2006/05/01 0.1 new
*-----------------------------------------------------------------------------*/
#ifndef RTKCMN_H
#define RTKCMN_H
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <time.h>
#include <ieeefp.h>						/* C99 */
#ifdef MKL
#include <mkl_types.h>
#include <mkl_lapack.h>
#include <mkl_blas.h>
#endif

#ifdef DEBUG
#define trace_ printf
#else
#define trace_ 1?(void)0:printf
#endif

#define PI			3.1415926535897932	/* pi */
#define D2R			(PI/180.0)			/* deg to rad */
#define R2D			(180.0/PI)			/* rad to deg */
#define C			299792458.0			/* speed of light in vacuum (m/s) */
#define MU			3.986005E14			/* earth gravity constant (WGS84) */
#define OMGE		7.2921151467E-5		/* earth rotation rate (WGS84) (rad/s) */
#define RE			6378137.0			/* earth semimajor axis (WGS84) (m) */
#define FE			(1.0/298.257223563)	/* earth flatterness (WGS84) */
#define FREQ1		1.57542E9			/* gps L1 frequency (Hz) */
#define FREQ2		1.22760E9			/* gps L2 frequency (Hz) */

#define MAXDT		0.05				/* max difference of epoch (sec) */
#define MAXSAT		32					/* max satellite number */
#define MAXOBS		32					/* max number of obs at an epoch*/

#define NAN			nan("")				/* C99 */
#define ISNAN(x)	isnan(x)			/* C99 */
#define ABS(x)		((x)<0?-(x):(x))
#define SQR(x)		((x)*(x))
#define MIN(x,y)	((x)<(y)?(x):(y))
#define MAX(x,y)	((x)>(y)?(x):(y))
#define DOT(x,y)	((x)[0]*(y)[0]+(x)[1]*(y)[1]+(x)[2]*(y)[2])
#define NORM(x)		sqrt(DOT(x,x))
#define MAT(n)		(double *)malloc(sizeof(double)*(n))

typedef struct {		/* time struct */
	time_t time;		/* time (sec) expressed by standard time_t */
	double sec;			/* flaction of sec */
} gtime_t;

typedef struct {		/* observation data record */
	gtime_t time;		/* receiver sampling time */
	int sat,rcv;		/* satellite/receiver number */
	double L[2],P[2];	/* observation data L1/L2/C1/P2 */
	short LLI[2];		/* loss of lock indicator L1/L2 */
} obsd_t;

typedef struct {		/* observation data */
	int n;				/* number of obervation data records */
	int siz;			/* number of memory allocation */
	obsd_t *data;		/* obervation data records */
} obs_t;

typedef struct {		/* navigation message record */
	int sat;			/* satellite number */
	int iode,iodc;		/* IODE,IODC */
	int sva,svh;		/* sv accuracy, sv health */
	gtime_t toe,toc;	/* Toe,Toc */
	double A,e,i0,OMG0,omg,M0,deln,OMGd,idot,crc,crs,cuc,cus,cic,cis,toes;
						/* sv ephemeris parameters */
	double f0,f1,f2,tgd; /* sv clock parameters */
} navm_t;

typedef struct {		/* navigation messages */
	int n;				/* number of navigation message records */
	int siz;			/* number of memory allocation */
	navm_t *msg;		/* navigation message records */
	double ion[8];		/* iono model parameters {a0,a1,a2,a3,b0,b1,b2,b3} */
	double utc[4];		/* delta-utc parameters */
} nav_t;

/* callback function types */
typedef int (* infunc_t)(obsd_t **, nav_t **);
typedef void (* outfunc_t)(gtime_t, double *);

/* lapack/blas prototypes */
#ifdef MKL
#define dsymm_		dsymm
#define dgemm_		dgemm
#define dgesv_		dgesv
#define dgels_		dgels
#define dpotrf_		dpotrf
#define dpotri_		dpotri
#define dgetrf_		dgetrf
#define dgetrs_		dgetrs
#else
extern void dsymm_(char *, char *, int *, int *, double *, double *, int *,
				   double *, int *, double *, double *, int *);
extern void dgemm_(char *, char *, int *, int *, int *, double *, const double *,
				   int *, const double *, int *, double *, double *, int *);
extern void dgesv_(int *, int *, double *, int *, int *, double *, int *, int *);
extern void dgels_(char *, int *, int *, int *, double *, int *, double *,
				   int *, double *, int *, int *);
extern void dpotrf_(char *, int *, double *, int *, int *);
extern void dpotri_(char *, int *, double *, int *, int *);
extern void dgetrf_(int *, int *, double *, int *, int *, int *);
extern void dgetrs_(char *, int *, int *, double *, int *, int *, double *,
				    int *, int *);
#endif /* MKL */

/* rtk common function prototypes */
extern gtime_t epoch2time(const double *ep);
extern gtime_t gpst2time(int week, double sec);
extern double time2gpst(gtime_t t, int *week);
extern gtime_t timeadd(gtime_t t, double sec);
extern double timediff(gtime_t t1, gtime_t t2);
extern void time2str(gtime_t t, char *str, int n);
extern int str2time(const char *s, int i, int n, gtime_t *t);
extern double str2num(const char *s, int i, int n);
extern void ecef2pos(const double *r, double *pos);
extern void pos2ecef(const double *pos, double *r);
extern void tolocal(const double *pos, const double *r, double *e);
extern void satpos(const obsd_t *obs, int n, const nav_t *nav, double *rs,
				   double *dts);
extern double geodist(const double *rs, const double *rr, double *e);
extern void satazel(const double *pos, const double *e, double *azel);
extern double ionmodel(gtime_t t, const double *ion, const double *pos,
					   const double *azel);
extern double tropmodel(const double *pos, const double *azel);
extern int lsq(double *H, double *y, int n, int m, double *x);
extern int filter(double *x, double *P, double *H, double *v, const double *R,
				  int n, int m);
extern void transx(const double *x, const double *P, const double *A, int n,
				   int m, double *y, double *Q);
extern void printm(const double *A, int n, int m, int p, int q);

#endif /* RTKCMN_H */
